/**
 * hermes antispam proxy
 * Copyright (C) 2006, 2007 Juan José Gutiérrez de Quevedo <juanjo@gutierrezdequevedo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @author Juan José Gutiérrez de Quevedo <juanjo@gutierrezdequevedo.com>
 */
#ifndef UTILS_H
#define UTILS_H

#include "hermes.h"
#include <string>
#include <string_view>
#include <sstream>
#include <iostream>
#include <dirent.h>
#include <time.h>

#ifndef WIN32
  #include <pwd.h>
  #include <grp.h>
#endif //WIN32

#include "Database.h"
#include "Socket.h"

using std::string;
using std::stringstream;
using std::list;

#ifdef WIN32
#define sleep(x) Sleep(1000*(x))
#endif //WIN32

/*#ifndef MAX
#define MAX(x,y) (((x)>(y))?(x):(y))
#endif //MAX*/

/**
 * this class implements common utilities
 */
class Utils
{
  public:
    //string utilities
    static string strtolower(std::string_view);
    static string trim(std::string_view);
    static string inttostr(int);
    static string ulongtostr(unsigned long);

    //email-related utilities
    static string getmail(const string&);
    static string getdomain(const string&);
    static string reverseip(const string&);

    //spam-related utilities (TODO: move to a different class)
    static bool greylist(const string& dbfile, string& ip, string& p_from, string& p_to);
    static bool listed_on_dns_lists(const list<string>& dns_domains, unsigned char percentage, const string& ip);
    static bool whitelisted(const string& dbfile, string& ip);
    static bool blacklisted(const string& dbfile, string& ip, string& to);

    #ifndef WIN32
    //posix-utils
    static int usertouid(const string& user);
    static int grouptogid(const string& groupname);
    #endif //WIN32

    //misc
    static string get_canonical_filename(const string& file);
    static bool file_exists(const string& file);
    static bool dir_exists(const string& dir);
    static string errnotostrerror(int);
    static string rfc2821_date(time_t *timestamp=NULL);
    static string gethostname();
    static void write_pid(const string& file, pid_t pid);
    static string gethostname(int socket);
};

#endif //UTILS_H
